@@ -20,9 +20,16 @@
 #ifndef FIREJAIL_H
 #define FIREJAIL_H
 #include "../include/common.h"
+#include "../include/euid_common.h"
+
 
 // filesystem
+#define RUN_FIREJAIL_BASEDIR	"/run"
 #define RUN_FIREJAIL_DIR	"/run/firejail"
+#define RUN_FIREJAIL_NAME_DIR	"/run/firejail/name"
+#define RUN_FIREJAIL_X11_DIR	"/run/firejail/x11"
+#define RUN_FIREJAIL_NETWORK_DIR	"/run/firejail/network"
+#define RUN_FIREJAIL_BANDWIDTH_DIR	"/run/firejail/bandwidth"
 #define RUN_NETWORK_LOCK_FILE	"/run/firejail/firejail.lock"
 #define RUN_RO_DIR	"/run/firejail/firejail.ro.dir"
 #define RUN_RO_FILE	"/run/firejail/firejail.ro.file"
@@ -40,6 +47,7 @@
 #define RUN_PULSE_DIR	"/run/firejail/mnt/pulse"
 #define RUN_DEVLOG_FILE	"/run/firejail/mnt/devlog"
 
+#define RUN_WHITELIST_X11_DIR	"/run/firejail/mnt/orig-x11"
 #define RUN_WHITELIST_HOME_DIR	"/run/firejail/mnt/orig-home"	// default home directory masking
 #define RUN_WHITELIST_HOME_USER_DIR	"/run/firejail/mnt/orig-home-user"	// home directory whitelisting
 #define RUN_WHITELIST_TMP_DIR	"/run/firejail/mnt/orig-tmp"
@@ -60,56 +68,10 @@
 #define RUN_FSLOGGER_FILE		"/run/firejail/mnt/fslogger"
 
 // profiles
-#define DEFAULT_USER_PROFILE	"generic"
+#define DEFAULT_USER_PROFILE	"default"
 #define DEFAULT_ROOT_PROFILE	"server"
 #define MAX_INCLUDE_LEVEL 6		// include levels in profile files
 
-#define ASSERT_PERMS(file, uid, gid, mode) \
-	do { \
-		assert(file);\
-		struct stat s;\
-		if (stat(file, &s) == -1) errExit("stat");\
-		assert(s.st_uid == uid);\
-		assert(s.st_gid == gid);\
-		assert((s.st_mode & 07777) == (mode));\
-	} while (0)
-#define ASSERT_PERMS_FD(fd, uid, gid, mode) \
-	do { \
-		struct stat s;\
-		if (fstat(fd, &s) == -1) errExit("stat");\
-		assert(s.st_uid == uid);\
-		assert(s.st_gid == gid);\
-		assert((s.st_mode & 07777) == (mode));\
-	} while (0)
-#define ASSERT_PERMS_STREAM(file, uid, gid, mode) \
-	do { \
-		int fd = fileno(file);\
-		if (fd == -1) errExit("fileno");\
-		ASSERT_PERMS_FD(fd, uid, gid, (mode));\
-	} while (0)
-
-#define SET_PERMS_FD(fd, uid, gid, mode) \
-	do { \
-		if (fchmod(fd, (mode)) == -1)	errExit("chmod");\
-		if (fchown(fd, uid, gid) == -1) errExit("chown");\
-	} while (0)
-#define SET_PERMS_STREAM(stream, uid, gid, mode) \
-	do { \
-		int fd = fileno(stream);\
-		if (fd == -1) errExit("fileno");\
-		SET_PERMS_FD(fd, uid, gid, (mode));\
-	} while (0)
-#define SET_PERMS_STREAM_NOERR(stream, uid, gid, mode) \
-	do { \
-		int fd = fileno(stream);\
-		if (fd == -1) continue;\
-		int rv = fchmod(fd, (mode));\
-		(void) rv;\
-		rv = fchown(fd, uid, gid);\
-		(void) rv;\
-	} while (0)
-
-
 // main.c
 typedef struct bridge_t {
 	// on the host
@@ -203,13 +165,15 @@ typedef struct config_t {
 	unsigned rlimit_fsize;
 	unsigned rlimit_sigpending;
 	
-	// cpu affinity and control groups
+	// cpu affinity, nice and control groups
 	uint32_t cpus;
+	int nice;
 	char *cgroup;
 	
 
 	// command line
 	char *command_line;
+	char *window_title;
 	char *command_name;
 	char *shell;
 	char **original_argv;
@@ -218,6 +182,19 @@ typedef struct config_t {
 } Config;
 extern Config cfg;
 
+static inline Bridge *last_bridge_configured(void) {
+	if (cfg.bridge3.configured)
+		return &cfg.bridge3;
+	else if (cfg.bridge2.configured)
+		return &cfg.bridge2;
+	else if (cfg.bridge1.configured)
+		return &cfg.bridge1;
+	else if (cfg.bridge0.configured)
+		return &cfg.bridge0;
+	else
+		return NULL;
+}
+
 static inline int any_bridge_configured(void) {
 	if (cfg.bridge0.configured || cfg.bridge1.configured || cfg.bridge2.configured || cfg.bridge3.configured)
 		return 1;
@@ -231,6 +208,7 @@ static inline int any_interface_configured(void) {
 	else
 		return 0;
 }
+void clear_run_files(pid_t pid);
 
 extern int arg_private;		// mount private /home
 extern int arg_debug;		// print debug messages
@@ -259,6 +237,7 @@ extern int arg_rlimit_nproc;	// rlimit nproc
 extern int arg_rlimit_fsize;	// rlimit fsize
 extern int arg_rlimit_sigpending;// rlimit sigpending
 extern int arg_nogroups;	// disable supplementary groups
+extern int arg_nonewprivs;	// set the NO_NEW_PRIVS prctl
 extern int arg_noroot;		// create a new user namespace and disable root user
 extern int arg_netfilter;	// enable netfilter
 extern int arg_netfilter6;	// enable netfilter6
@@ -276,6 +255,10 @@ extern int arg_nosound;	// disable sound
 extern int arg_quiet;		// no output for scripting
 extern int arg_join_network;	// join only the network namespace
 extern int arg_join_filesystem;	// join only the mount namespace
+extern int arg_nice;		// nice value configured
+extern int arg_ipc;		// enable ipc namespace
+extern int arg_writable_etc;	// writable etc
+extern int arg_writable_var;	// writable var
 
 extern int parent_to_child_fds[2];
 extern int child_to_parent_fds[2];
@@ -298,6 +281,7 @@ void net_configure_veth_pair(Bridge *br, const char *ifname, pid_t child);
 void net_check_cfg(void);
 void net_dns_print_name(const char *name);
 void net_dns_print(pid_t pid);
+void network_main(pid_t child);
 
 // network.c
 void net_if_up(const char *ifname);
@@ -333,7 +317,7 @@ void fs_basic_fs(void);
 void fs_overlayfs(void);
 // chroot into an existing directory; mount exiting /dev and update /etc/resolv.conf
 void fs_chroot(const char *rootdir);
-void fs_check_chroot_dir(const char *rootdir);
+int fs_check_chroot_dir(const char *rootdir);
 void fs_private_tmp(void);
 
 // profile.c
@@ -358,8 +342,8 @@ void netstats(void);
 void usage(void);
 
 // join.c
-void join(pid_t pid, const char *homedir, int argc, char **argv, int index);
-void join_name(const char *name, const char *homedir, int argc, char **argv, int index);
+void join(pid_t pid, int argc, char **argv, int index);
+void join_name(const char *name, int argc, char **argv, int index);
 void shut(pid_t pid);
 void shut_name(const char *name);
 
@@ -389,8 +373,6 @@ void logmsg(const char *msg);
 void logargs(int argc, char **argv) ;
 void logerr(const char *msg);
 int copy_file(const char *srcname, const char *destname);
-void copy_file_as_user(const char *srcname, const char *destname, uid_t uid, gid_t gid, mode_t mode);
-void touch_file_as_user(const char *fname, uid_t uid, gid_t gid, mode_t mode);
 int is_dir(const char *fname);
 int is_link(const char *fname);
 char *line_remove_spaces(const char *buf);
@@ -405,8 +387,8 @@ char *expand_home(const char *path, const char* homedir);
 const char *gnu_basename(const char *path);
 uid_t pid_get_uid(pid_t pid);
 void invalid_filename(const char *fname);
-int remove_directory(const char *path);
-void flush_stdin(void);
+uid_t get_tty_gid(void);
+uid_t get_audio_gid(void);
 
 // fs_var.c
 void fs_var_log(void);	// mounting /var/log
@@ -473,6 +455,8 @@ void read_cpu_list(const char *str);
 void set_cpu_affinity(void);
 void load_cpu(const char *fname);
 void save_cpu(void);
+void cpu_print_filter_name(const char *name);
+void cpu_print_filter(pid_t pid);
 
 // cgroup.c
 void save_cgroup(void);
@@ -488,13 +472,11 @@ void netfilter(const char *fname);
 void netfilter6(const char *fname);
 
 // bandwidth.c
-void shm_create_firejail_dir(void);
-void bandwidth_shm_del_file(pid_t pid);
-void bandwidth_shm_set(pid_t pid, const char *dev, int down, int up);
+void bandwidth_del_run_file(pid_t pid);
 void bandwidth_name(const char *name, const char *command, const char *dev, int down, int up);
 void bandwidth_pid(pid_t pid, const char *command, const char *dev, int down, int up);
-void network_shm_del_file(pid_t pid);
-void network_shm_set_file(pid_t pid);
+void network_del_run_file(pid_t pid);
+void network_set_run_file(pid_t pid);
 
 // fs_etc.c
 void fs_check_etc_list(void);
@@ -555,5 +537,43 @@ void run_symlink(int argc, char **argv);
 // user.c
 void check_user(int argc, char **argv);
 
+// paths.c
+char **build_paths(void);
+
+// fs_mkdir.c
+void fs_mkdir(const char *name);
+
+// x11.c
+void fs_x11(void);
+int x11_display(void);
+void x11_start(int argc, char **argv);
+void x11_start_xpra(int argc, char **argv);
+void x11_start_xephyr(int argc, char **argv);
+extern char *xephyr_screen;
+
+// ls.c
+#define SANDBOX_FS_LS 0
+#define SANDBOX_FS_GET 1
+void sandboxfs_name(int op, const char *name, const char *path);
+void sandboxfs(int op, pid_t pid, const char *patqh);
+
+// checkcfg.c
+#define CFG_FILE_TRANSFER 0
+#define CFG_X11 1
+#define CFG_BIND 2
+#define CFG_USERNS 3
+#define CFG_CHROOT 4
+#define CFG_SECCOMP 5
+#define CFG_NETWORK 6
+#define CFG_RESTRICTED_NETWORK 7
+#define CFG_FORCE_NONEWPRIVS 8
+#define CFG_MAX 9 // this should always be the last entry
+int checkcfg(int val);
+
+// fs_rdwr.c
+void fs_rdwr_add(const char *path);
+void fs_rdwr(void);
+
+
 #endif
 
