@@ -60,6 +60,26 @@ static void create_empty_file(void) {
 void fs_build_firejail_dir(void) {
 	struct stat s;
 
+	// CentOS 6 doesn't have /run directory
+	if (stat(RUN_FIREJAIL_BASEDIR, &s)) {
+		if (arg_debug)
+			printf("Creating %s directory\n", RUN_FIREJAIL_BASEDIR);
+		/* coverity[toctou] */
+		int rv = mkdir(RUN_FIREJAIL_BASEDIR, 0755);
+		if (rv == -1)
+			errExit("mkdir");
+		if (chown(RUN_FIREJAIL_BASEDIR, 0, 0) < 0)
+			errExit("chown");
+		if (chmod(RUN_FIREJAIL_BASEDIR, 0755) < 0)
+			errExit("chmod");
+	}
+	else { // check /tmp/firejail directory belongs to root end exit if doesn't!
+		if (s.st_uid != 0 || s.st_gid != 0) {
+			fprintf(stderr, "Error: non-root %s directory, exiting...\n", RUN_FIREJAIL_DIR);
+			exit(1);
+		}
+	}
+
 	if (stat(RUN_FIREJAIL_DIR, &s)) {
 		if (arg_debug)
 			printf("Creating %s directory\n", RUN_FIREJAIL_DIR);
@@ -72,11 +92,50 @@ void fs_build_firejail_dir(void) {
 		if (chmod(RUN_FIREJAIL_DIR, 0755) < 0)
 			errExit("chmod");
 	}
-	else { // check /tmp/firejail directory belongs to root end exit if doesn't!
-		if (s.st_uid != 0 || s.st_gid != 0) {
-			fprintf(stderr, "Error: non-root %s directory, exiting...\n", RUN_FIREJAIL_DIR);
-			exit(1);
-		}
+	
+	if (stat(RUN_FIREJAIL_NETWORK_DIR, &s)) {
+		if (arg_debug)
+			printf("Creating %s directory\n", RUN_FIREJAIL_NETWORK_DIR);
+		
+		if (mkdir(RUN_FIREJAIL_NETWORK_DIR, 0755) == -1)
+			errExit("mkdir");
+		if (chown(RUN_FIREJAIL_NETWORK_DIR, 0, 0) < 0)
+			errExit("chown");
+		if (chmod(RUN_FIREJAIL_NETWORK_DIR, 0755) < 0)
+			errExit("chmod");
+	}
+	
+	if (stat(RUN_FIREJAIL_BANDWIDTH_DIR, &s)) {
+		if (arg_debug)
+			printf("Creating %s directory\n", RUN_FIREJAIL_BANDWIDTH_DIR);
+		if (mkdir(RUN_FIREJAIL_BANDWIDTH_DIR, 0755) == -1)
+			errExit("mkdir");
+		if (chown(RUN_FIREJAIL_BANDWIDTH_DIR, 0, 0) < 0)
+			errExit("chown");
+		if (chmod(RUN_FIREJAIL_BANDWIDTH_DIR, 0755) < 0)
+			errExit("chmod");
+	}
+		
+	if (stat(RUN_FIREJAIL_NAME_DIR, &s)) {
+		if (arg_debug)
+			printf("Creating %s directory\n", RUN_FIREJAIL_NAME_DIR);
+		if (mkdir(RUN_FIREJAIL_NAME_DIR, 0755) == -1)
+			errExit("mkdir");
+		if (chown(RUN_FIREJAIL_NAME_DIR, 0, 0) < 0)
+			errExit("chown");
+		if (chmod(RUN_FIREJAIL_NAME_DIR, 0755) < 0)
+			errExit("chmod");
+	}
+	
+	if (stat(RUN_FIREJAIL_X11_DIR, &s)) {
+		if (arg_debug)
+			printf("Creating %s directory\n", RUN_FIREJAIL_X11_DIR);
+		if (mkdir(RUN_FIREJAIL_X11_DIR, 0755) == -1)
+			errExit("mkdir");
+		if (chown(RUN_FIREJAIL_X11_DIR, 0, 0) < 0)
+			errExit("chown");
+		if (chmod(RUN_FIREJAIL_X11_DIR, 0755) < 0)
+			errExit("chmod");
 	}
 	
 	create_empty_dir();
@@ -119,6 +178,7 @@ void fs_build_mnt_dir(void) {
 		if (mount("tmpfs", RUN_MNT_DIR, "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
 			errExit("mounting /tmp/firejail/mnt");
 		tmpfs_mounted = 1;
+		fs_logger2("tmpfs", RUN_MNT_DIR);
 	}
 }
 
@@ -189,7 +249,7 @@ static void disable_file(OPERATION op, const char *filename) {
 	char* fname = realpath(filename, NULL);
 	if (fname == NULL && errno != EACCES) {
 		if (arg_debug)
-			printf("Warning: %s is an invalid file, skipping...\n", filename);
+			printf("Warning (realpath): %s is an invalid file, skipping...\n", filename);
 		return;
 	}
 	if (fname == NULL && errno == EACCES) {
@@ -216,7 +276,7 @@ static void disable_file(OPERATION op, const char *filename) {
 		}
 		else {
 			if (arg_debug)
-				printf("Warning: %s is an invalid file, skipping...\n", filename);
+				printf("Warning (blacklisting): %s is an invalid file, skipping...\n", filename);
 		}
 				
 		return;
@@ -224,6 +284,8 @@ static void disable_file(OPERATION op, const char *filename) {
 	
 	// if the file is not present, do nothing
 	struct stat s;
+	if (fname == NULL)
+		return;
 	if (stat(fname, &s) == -1) {
 		if (arg_debug)
 			printf("Warning: %s does not exist, skipping...\n", fname);
@@ -281,7 +343,7 @@ static void disable_file(OPERATION op, const char *filename) {
 			if (chown(fname, s.st_uid, s.st_gid) == -1)
 				errExit("mounting tmpfs chmod");
 			last_disable = SUCCESSFUL;
-			fs_logger2("mount tmpfs on", fname);
+			fs_logger2("tmpfs", fname);
 		}
 		else
 			printf("Warning: %s is not a directory; cannot mount a tmpfs on top of it.\n", fname);
@@ -373,12 +435,12 @@ void fs_blacklist(void) {
 			}
 			struct stat s;
 			if (stat(dname1, &s) == -1) {
-				fprintf(stderr, "Error: cannot find directories for bind command\n");
+				fprintf(stderr, "Error: cannot find %s for bind command\n", dname1);
 				entry = entry->next;
 				continue;
 			}
 			if (stat(dname2, &s) == -1) {
-				fprintf(stderr, "Error: cannot find directories for bind command\n");
+				fprintf(stderr, "Error: cannot find %s for bind command\n", dname2);
 				entry = entry->next;
 				continue;
 			}
@@ -403,13 +465,11 @@ void fs_blacklist(void) {
 		// Process noblacklist command
 		if (strncmp(entry->data, "noblacklist ", 12) == 0) {
 			if (noblacklist_c >= noblacklist_m) {
-                                noblacklist_m *= 2;
-                                noblacklist = realloc(noblacklist, sizeof(*noblacklist) * noblacklist_m);
-                                if (noblacklist == NULL)
-                                        errExit("failed increasing memory for noblacklist entries");
-			}
-			else
-				noblacklist[noblacklist_c++] = expand_home(entry->data + 12, homedir);
+			noblacklist_m *= 2;
+			noblacklist = realloc(noblacklist, sizeof(*noblacklist) * noblacklist_m);
+			if (noblacklist == NULL)
+				errExit("failed increasing memory for noblacklist entries");}
+			noblacklist[noblacklist_c++] = expand_home(entry->data + 12, homedir);
 			entry = entry->next;
 			continue;
 		}
@@ -441,18 +501,19 @@ void fs_blacklist(void) {
 		char *new_name = expand_home(ptr, homedir);
 		ptr = new_name;
 
-		// expand path macro - look for the file in /bin, /usr/bin, /sbin and  /usr/sbin directories
+		// expand path macro - look for the file in /usr/local/bin,  /usr/local/sbin, /bin, /usr/bin, /sbin and  /usr/sbin directories
 		if (ptr) {
 			if (strncmp(ptr, "${PATH}", 7) == 0) {
 				char *fname = ptr + 7;
 				size_t fname_len = strlen(fname);
-				char **path, *paths[] = {"/bin", "/sbin", "/usr/bin", "/usr/sbin", NULL};
-				for (path = &paths[0]; *path; path++) {
-					char newname[strlen(*path) + fname_len + 1];
-					sprintf(newname, "%s%s", *path, fname);
+				char **paths = build_paths(); //{"/usr/local/bin", "/usr/local/sbin", "/bin", "/usr/bin/", "/sbin", "/usr/sbin", NULL};
+				int i = 0;
+				while (paths[i] != NULL) {
+					char *path = paths[i];
+					i++;
+					char newname[strlen(path) + fname_len + 1];
+					sprintf(newname, "%s%s", path, fname);
 					globbing(op, newname, (const char**)noblacklist, noblacklist_c);
-					if  (last_disable == SUCCESSFUL)
-						break;
 				}
 			}
 			else
@@ -531,12 +592,14 @@ void fs_proc_sys_dev_boot(void) {
 	/* Mount a version of /sys that describes the network namespace */
 	if (arg_debug)
 		printf("Remounting /sys directory\n");
-	if (umount2("/sys", MNT_DETACH) < 0) 
+	if (umount2("/sys", MNT_DETACH) < 0)
 		fprintf(stderr, "Warning: failed to unmount /sys\n");
-	if (mount("sysfs", "/sys", "sysfs", MS_RDONLY|MS_NOSUID|MS_NOEXEC|MS_NODEV|MS_REC, NULL) < 0)
-		fprintf(stderr, "Warning: failed to mount /sys\n");
-	else
-		fs_logger("remount /sys");
+	else {
+		if (mount("sysfs", "/sys", "sysfs", MS_RDONLY|MS_NOSUID|MS_NOEXEC|MS_NODEV|MS_REC, NULL) < 0)
+			fprintf(stderr, "Warning: failed to mount /sys\n");
+		else
+			fs_logger("remount /sys");
+	}
 		
 	if (stat("/sys/firmware", &s) == 0) {
 		disable_file(BLACKLIST_FILE, "/sys/firmware");
@@ -645,25 +708,41 @@ static void disable_firejail_config(void) {
 		if (stat(fname, &s) == 0)
 			disable_file(BLACKLIST_FILE, fname);
 	}
-
-	
 	
 	free(fname);
+	
+	// disable run time information
+	if (stat(RUN_FIREJAIL_NETWORK_DIR, &s) == 0)
+		disable_file(BLACKLIST_FILE, RUN_FIREJAIL_NETWORK_DIR);
+	if (stat(RUN_FIREJAIL_BANDWIDTH_DIR, &s) == 0)
+		disable_file(BLACKLIST_FILE, RUN_FIREJAIL_BANDWIDTH_DIR);
+	if (stat(RUN_FIREJAIL_NAME_DIR, &s) == 0)
+		disable_file(BLACKLIST_FILE, RUN_FIREJAIL_NAME_DIR);
+	if (stat(RUN_FIREJAIL_X11_DIR, &s) == 0)
+		disable_file(BLACKLIST_FILE, RUN_FIREJAIL_X11_DIR);
 }
 
 
 // build a basic read-only filesystem
 void fs_basic_fs(void) {
 	if (arg_debug)
-		printf("Mounting read-only /bin, /sbin, /lib, /lib32, /lib64, /usr, /etc, /var\n");
+		printf("Mounting read-only /bin, /sbin, /lib, /lib32, /lib64, /usr");
+	if (!arg_writable_etc) {
+		fs_rdonly("/etc");
+		if (arg_debug) printf(", /etc");
+	}
+	if (!arg_writable_var) {
+		fs_rdonly("/var");
+		if (arg_debug) printf(", /var");
+	}
+	if (arg_debug) printf("\n");
 	fs_rdonly("/bin");
 	fs_rdonly("/sbin");
 	fs_rdonly("/lib");
 	fs_rdonly("/lib64");
 	fs_rdonly("/lib32");
+	fs_rdonly("/libx32");
 	fs_rdonly("/usr");
-	fs_rdonly("/etc");
-	fs_rdonly("/var");
 
 	// update /var directory in order to support multiple sandboxes running on the same root directory
 	if (!arg_private_dev)
@@ -678,7 +757,16 @@ void fs_basic_fs(void) {
 	// don't leak user information
 	restrict_users();
 	
-	disable_firejail_config();
+	// when starting as root, firejail config is not disabled;
+	// this mode could be used to install and test new software by chaining
+	// firejail sandboxes (firejail --force)
+	if (getuid() != 0)
+		disable_firejail_config();
+	else
+		fprintf(stderr, "Warning: masking /etc/firejail disabled when starting the sandbox as root\n");
+		
+	if (getuid() == 0)
+		fs_rdwr();
 }
 
 
@@ -794,9 +882,67 @@ void fs_overlayfs(void) {
 	else { // kernel 3.18 or newer
 		if (asprintf(&option, "lowerdir=/,upperdir=%s,workdir=%s", odiff, owork) == -1)
 			errExit("asprintf");
-//printf("option #%s#\n", option);			
 		if (mount("overlay", oroot, "overlay", MS_MGC_VAL, option) < 0)
 			errExit("mounting overlayfs");
+			
+		//***************************
+		// issue #263 start code
+		// My setup has a separate mount point for /home. When the overlay is mounted,
+		// the overlay does not contain the original /home contents. 
+		// I added code to create a second overlay for /home if the overlay home dir is empty and this seems to work
+		// @dshmgh, Jan 2016
+		{
+			char *overlayhome;
+			struct stat s;
+			char *hroot;
+			char *hdiff;
+			char *hwork;
+		
+			// dons add debug
+			if (arg_debug) printf ("DEBUG: chroot dirs are oroot %s  odiff %s  owork %s\n",oroot,odiff,owork);
+		
+			// BEFORE NEXT, WE NEED TO TEST IF /home has any contents or do we need to mount it?
+			// must create var for oroot/cfg.homedir
+			if (asprintf(&overlayhome,"%s%s",oroot,cfg.homedir) == -1)
+				errExit("asprintf");
+			if (arg_debug) printf ("DEBUG: overlayhome var holds ##%s##\n",overlayhome);
+		
+			// if no homedir in overlay -- create another overlay for /home
+			if (stat(overlayhome, &s) == -1) {
+		
+				if(asprintf(&hroot, "%s/oroot/home", RUN_MNT_DIR) == -1)
+					errExit("asprintf");
+		
+				if(asprintf(&hdiff, "%s/hdiff", basedir) == -1)
+					errExit("asprintf");
+				if (mkdir(hdiff, S_IRWXU | S_IRWXG | S_IRWXO))
+					errExit("mkdir");
+				if (chown(hdiff, 0, 0) < 0)
+					errExit("chown");
+				if (chmod(hdiff, S_IRWXU  | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) < 0)
+					errExit("chmod");
+		
+				if(asprintf(&hwork, "%s/hwork", basedir) == -1)
+					errExit("asprintf");
+				if (mkdir(hwork, S_IRWXU | S_IRWXG | S_IRWXO))
+					errExit("mkdir");
+				if (chown(hwork, 0, 0) < 0)
+					errExit("chown");
+				if (chmod(hwork, S_IRWXU  | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) < 0)
+					errExit("chmod");
+		
+				// no homedir in overlay so now mount another overlay for /home
+				if (asprintf(&option, "lowerdir=/home,upperdir=%s,workdir=%s", hdiff, hwork) == -1)
+					errExit("asprintf");
+				if (mount("overlay", hroot, "overlay", MS_MGC_VAL, option) < 0)
+					errExit("mounting overlayfs for mounted home directory");
+		
+				printf("OverlayFS for /home configured in %s directory\n", basedir);
+			} // stat(overlayhome)
+			free(overlayhome);
+		}
+		// issue #263 end code
+		//***************************
 	}
 	printf("OverlayFS configured in %s directory\n", basedir);
 	
@@ -808,10 +954,22 @@ void fs_overlayfs(void) {
 		errExit("asprintf");
 	if (mount("/dev", dev, NULL, MS_BIND|MS_REC, NULL) < 0)
 		errExit("mounting /dev");
+	fs_logger("whitelist /dev");
+
+	// mount-bind run directory
+	if (arg_debug)
+		printf("Mounting /run\n");
+	char *run;
+	if (asprintf(&run, "%s/run", oroot) == -1)
+		errExit("asprintf");
+	if (mount("/run", run, NULL, MS_BIND|MS_REC, NULL) < 0)
+		errExit("mounting /run");
+	fs_logger("whitelist /run");
 
 	// chroot in the new filesystem
 	if (chroot(oroot) == -1)
 		errExit("chroot");
+
 	// update /var directory in order to support multiple sandboxes running on the same root directory
 	if (!arg_private_dev)
 		fs_dev_shm();
@@ -825,7 +983,13 @@ void fs_overlayfs(void) {
 	// don't leak user information
 	restrict_users();
 
-	disable_firejail_config();
+	// when starting as root, firejail config is not disabled;
+	// this mode could be used to install and test new software by chaining
+	// firejail sandboxes (firejail --force)
+	if (getuid() != 0)
+		disable_firejail_config();
+	else
+		fprintf(stderr, "Warning: masking /etc/firejail disabled when starting the sandbox as root\n");
 
 	// cleanup and exit
 	free(option);
@@ -837,36 +1001,18 @@ void fs_overlayfs(void) {
 
 #ifdef HAVE_CHROOT		
 // return 1 if error
-void fs_check_chroot_dir(const char *rootdir) {
+int fs_check_chroot_dir(const char *rootdir) {
+	EUID_ASSERT();
 	assert(rootdir);
 	struct stat s;
 	char *name;
 
-	if (strcmp(rootdir, "/tmp") == 0 || strcmp(rootdir, "/var/tmp") == 0) {
-		fprintf(stderr, "Error: invalid chroot directory\n");
-		exit(1);
-	}
-
-	// rootdir has to be owned by root
-	if (stat(rootdir, &s) != 0) {
-		fprintf(stderr, "Error: cannot find chroot directory\n");
-		exit(1);
-	}
-	if (s.st_uid != 0) {
-		fprintf(stderr, "Error: chroot directory should be owned by root\n");
-		exit(1);
-	}
-
 	// check /dev
 	if (asprintf(&name, "%s/dev", rootdir) == -1)
 		errExit("asprintf");
 	if (stat(name, &s) == -1) {
 		fprintf(stderr, "Error: cannot find /dev in chroot directory\n");
-		exit(1);
-	}
-	if (s.st_uid != 0) {
-		fprintf(stderr, "Error: chroot /dev directory should be owned by root\n");
-		exit(1);
+		return 1;
 	}
 	free(name);
 
@@ -875,11 +1021,7 @@ void fs_check_chroot_dir(const char *rootdir) {
 		errExit("asprintf");
 	if (stat(name, &s) == -1) {
 		fprintf(stderr, "Error: cannot find /var/tmp in chroot directory\n");
-		exit(1);
-	}
-	if (s.st_uid != 0) {
-		fprintf(stderr, "Error: chroot /var/tmp directory should be owned by root\n");
-		exit(1);
+		return 1;
 	}
 	free(name);
 	
@@ -888,54 +1030,29 @@ void fs_check_chroot_dir(const char *rootdir) {
 		errExit("asprintf");
 	if (stat(name, &s) == -1) {
 		fprintf(stderr, "Error: cannot find /proc in chroot directory\n");
-		exit(1);
-	}
-	if (s.st_uid != 0) {
-		fprintf(stderr, "Error: chroot /proc directory should be owned by root\n");
-		exit(1);
+		return 1;
 	}
 	free(name);
 	
-	// check /tmp
+	// check /proc
 	if (asprintf(&name, "%s/tmp", rootdir) == -1)
 		errExit("asprintf");
 	if (stat(name, &s) == -1) {
 		fprintf(stderr, "Error: cannot find /tmp in chroot directory\n");
-		exit(1);
-	}
-	if (s.st_uid != 0) {
-		fprintf(stderr, "Error: chroot /tmp directory should be owned by root\n");
-		exit(1);
+		return 1;
 	}
 	free(name);
-
-	// check /etc
-	if (asprintf(&name, "%s/etc", rootdir) == -1)
+	
+	// check /bin/bash
+	if (asprintf(&name, "%s/bin/bash", rootdir) == -1)
 		errExit("asprintf");
 	if (stat(name, &s) == -1) {
-		fprintf(stderr, "Error: cannot find /etc in chroot directory\n");
-		exit(1);
-	}
-	if (s.st_uid != 0) {
-		fprintf(stderr, "Error: chroot /etc directory should be owned by root\n");
-		exit(1);
+		fprintf(stderr, "Error: cannot find /bin/bash in chroot directory\n");
+		return 1;
 	}
 	free(name);
 
-	// check /etc/resolv.conf
-	if (asprintf(&name, "%s/etc/resolv.conf", rootdir) == -1)
-		errExit("asprintf");
-	if (stat(name, &s) == 0) {
-		if (s.st_uid != 0) {
-			fprintf(stderr, "Error: chroot /etc/resolv.conf should be owned by root\n");
-			exit(1);
-		}
-	}
-	if (is_link(name)) {
-		fprintf(stderr, "Error: invalid %s file\n", name);
-		exit(1);
-	}
-	free(name);
+	return 0;	
 }
 
 // chroot into an existing directory; mount exiting /dev and update /etc/resolv.conf
@@ -960,10 +1077,6 @@ void fs_chroot(const char *rootdir) {
 	char *rundir;
 	if (asprintf(&rundir, "%s/run", rootdir) == -1)
 		errExit("asprintf");
-	if (is_link(rundir)) {
-		fprintf(stderr, "Error: invalid run directory inside chroot\n");
-		exit(1);
-	}
 	if (!is_dir(rundir)) {
 		int rv = mkdir(rundir, 0755);
 		(void) rv;
@@ -983,14 +1096,8 @@ void fs_chroot(const char *rootdir) {
 		fprintf(stderr, "Error: invalid %s file\n", fname);
 		exit(1);
 	}
-	if (copy_file("/etc/resolv.conf", fname) == -1) // root needed
+	if (copy_file("/etc/resolv.conf", fname) == -1)
 		fprintf(stderr, "Warning: /etc/resolv.conf not initialized\n");
-	else {
-		if (chown("/etc/resolv.conf", 0, 0) == -1)
-			errExit("chown");
-		if (chmod("/etc/resolv.conf", 0644) == -1)
-			errExit("chmod");
-	}
 		
 	// chroot into the new directory
 	if (arg_debug)
@@ -1013,7 +1120,13 @@ void fs_chroot(const char *rootdir) {
 	// don't leak user information
 	restrict_users();
 
-	disable_firejail_config();
+	// when starting as root, firejail config is not disabled;
+	// this mode could be used to install and test new software by chaining
+	// firejail sandboxes (firejail --force)
+	if (getuid() != 0)
+		disable_firejail_config();
+	else
+		fprintf(stderr, "Warning: masking /etc/firejail disabled when starting the sandbox as root\n");
 }
 #endif
 
@@ -1023,5 +1136,6 @@ void fs_private_tmp(void) {
 		printf("Mounting tmpfs on /tmp directory\n");
 	if (mount("tmpfs", "/tmp", "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=1777,gid=0") < 0)
 		errExit("mounting /tmp/firejail/mnt");
+	fs_logger2("tmpfs", "/tmp");
 }
 
