@@ -249,7 +249,7 @@ static void whitelist_path(ProfileEntry *entry) {
 	}
 	else {
 		if (arg_debug || arg_debug_whitelists) {
-			fprintf(stderr, "Warning: %s is an invalid file, skipping...\n", path);
+ 			fprintf(stderr, "Warning (whitelisting): %s is an invalid file, skipping...\n", path);
 		}
 		return;
 	}
@@ -267,17 +267,13 @@ static void whitelist_path(ProfileEntry *entry) {
 	
 	// process regular file
 	else {
-		if (access(path, R_OK)) {
-			// create an empty file
-			FILE *fp = fopen(path, "w");
-			if (!fp) {
-				fprintf(stderr, "Error: cannot create empty file in home directory\n");
-				exit(1);
-			}
-			fclose(fp);
+		// create an empty file
+		FILE *fp = fopen(path, "w");
+		if (!fp) {
+			fprintf(stderr, "Error: cannot create empty file in home directory\n");
+			exit(1);
 		}
-		else
-			return; // the file is already present
+		fclose(fp);
 	}
 	
 	// set file properties
@@ -340,6 +336,14 @@ void fs_whitelist(void) {
 		if (arg_debug)
 			fprintf(stderr, "Debug %d: new_name #%s#\n", __LINE__, new_name);
 
+		// valid path referenced to filesystem root
+		if (*new_name != '/') {
+			if (arg_debug)
+				fprintf(stderr, "Debug %d: \n", __LINE__);
+			goto errexit;
+		}
+
+
 		// extract the absolute path of the file
 		// realpath function will fail with ENOENT if the file is not found
 		char *fname = realpath(new_name, NULL);
@@ -353,19 +357,29 @@ void fs_whitelist(void) {
 				perror("realpath");
 			}
 			*entry->data = '\0';
+
+			// if 1 the file was not found; mount an empty directory
+			if (strncmp(new_name, cfg.homedir, strlen(cfg.homedir)) == 0) {
+				if(!arg_private)
+					home_dir = 1;
+			}
+			else if (strncmp(new_name, "/tmp/", 5) == 0)
+				tmp_dir = 1;
+			else if (strncmp(new_name, "/media/", 7) == 0)
+				media_dir = 1;
+			else if (strncmp(new_name, "/var/", 5) == 0)
+				var_dir = 1;
+			else if (strncmp(new_name, "/dev/", 5) == 0)
+				dev_dir = 1;
+			else if (strncmp(new_name, "/opt/", 5) == 0)
+				opt_dir = 1;
+
 			continue;
 		}
 		
-		// valid path referenced to filesystem root
-		if (*new_name != '/') {
-			if (arg_debug)
-				fprintf(stderr, "Debug %d: \n", __LINE__);
-			goto errexit;
-		}
-
 		// check for supported directories
 		if (strncmp(new_name, cfg.homedir, strlen(cfg.homedir)) == 0) {
-			// whitelisting home directory is disabled if --private or --private-home option is present
+			// whitelisting home directory is disabled if --private option is present
 			if (arg_private) {
 				if (arg_debug || arg_debug_whitelists)
 					printf("Removed whitelist path %s, --private option is present\n", entry->data);
@@ -408,7 +422,12 @@ void fs_whitelist(void) {
 			entry->var_dir = 1;
 			var_dir = 1;
 			// both path and absolute path are under /var
-			if (strncmp(fname, "/var/", 5) != 0) {
+			// exceptions: /var/run and /var/lock
+			if (strcmp(new_name, "/var/run")== 0)
+				;
+			else if (strcmp(new_name, "/var/lock")== 0)
+				;
+			else if (strncmp(fname, "/var/", 5) != 0) {
 				if (arg_debug)
 					fprintf(stderr, "Debug %d: fname #%s#\n", __LINE__, fname);
 				goto errexit;
@@ -502,7 +521,7 @@ void fs_whitelist(void) {
 			printf("Mounting tmpfs on /tmp directory\n");
 		if (mount("tmpfs", "/tmp", "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=1777,gid=0") < 0)
 			errExit("mounting tmpfs on /tmp");
-		fs_logger("mount tmpfs on /tmp");
+		fs_logger("tmpfs /tmp");
 	}
 	
 	// /media mountpoint
@@ -524,7 +543,7 @@ void fs_whitelist(void) {
 			printf("Mounting tmpfs on /media directory\n");
 		if (mount("tmpfs", "/media", "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
 			errExit("mounting tmpfs on /media");
-		fs_logger("mount tmpfs on /media");
+		fs_logger("tmpfs /media");
 	}
 
 	// /var mountpoint
@@ -546,7 +565,7 @@ void fs_whitelist(void) {
 			printf("Mounting tmpfs on /var directory\n");
 		if (mount("tmpfs", "/var", "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
 			errExit("mounting tmpfs on /var");
-		fs_logger("mount tmpfs on /var");
+		fs_logger("tmpfs /var");
 	}
 
 	// /dev mountpoint
@@ -568,7 +587,7 @@ void fs_whitelist(void) {
 			printf("Mounting tmpfs on /dev directory\n");
 		if (mount("tmpfs", "/dev", "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
 			errExit("mounting tmpfs on /dev");
-		fs_logger("mount tmpfs on /dev");
+		fs_logger("tmpfs /dev");
 	}
 
 	// /opt mountpoint
@@ -590,7 +609,7 @@ void fs_whitelist(void) {
 			printf("Mounting tmpfs on /opt directory\n");
 		if (mount("tmpfs", "/opt", "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
 			errExit("mounting tmpfs on /opt");
-		fs_logger("mount tmpfs on /opt");
+		fs_logger("tmpfs /opt");
 	}
 
 	// go through profile rules again, and interpret whitelist commands
@@ -604,21 +623,31 @@ void fs_whitelist(void) {
 
 //printf("here %d#%s#\n", __LINE__, entry->data);
 		// whitelist the real file
-		whitelist_path(entry);
-
-		// create the link if any
-		if (entry->link) {
-			// if the link is already there, do not bother
-			struct stat s;
-			if (stat(entry->link, &s) != 0) {
-				// create the path if necessary
-				mkpath(entry->link, s.st_mode);
-
-				int rv = symlink(entry->data + 10, entry->link);
-				if (rv)
-					fprintf(stderr, "Warning cannot create symbolic link %s\n", entry->link);
-				else if (arg_debug || arg_debug_whitelists)
-					printf("Created symbolic link %s -> %s\n", entry->link, entry->data + 10);
+		if (strcmp(entry->data, "whitelist /run") == 0 && 
+		    (strcmp(entry->link, "/var/run") == 0 || strcmp(entry->link, "/var/lock") == 0)) {
+			int rv = symlink(entry->data + 10, entry->link);
+			if (rv)
+				fprintf(stderr, "Warning cannot create symbolic link %s\n", entry->link);
+			else if (arg_debug || arg_debug_whitelists)
+				printf("Created symbolic link %s -> %s\n", entry->link, entry->data + 10);
+		}
+		else {
+			whitelist_path(entry);
+	
+			// create the link if any
+			if (entry->link) {
+				// if the link is already there, do not bother
+				struct stat s;
+				if (stat(entry->link, &s) != 0) {
+					// create the path if necessary
+					mkpath(entry->link, s.st_mode);
+	
+					int rv = symlink(entry->data + 10, entry->link);
+					if (rv)
+						fprintf(stderr, "Warning cannot create symbolic link %s\n", entry->link);
+					else if (arg_debug || arg_debug_whitelists)
+						printf("Created symbolic link %s -> %s\n", entry->link, entry->data + 10);
+				}
 			}
 		}
 
@@ -629,12 +658,42 @@ void fs_whitelist(void) {
 	if (home_dir) {
 		if (mount("tmpfs", RUN_WHITELIST_HOME_USER_DIR, "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
 			errExit("mount tmpfs");
+		fs_logger2("tmpfs", RUN_WHITELIST_HOME_USER_DIR);
 	}
 	
 	// mask the real /tmp directory, currently mounted on RUN_WHITELIST_TMP_DIR
 	if (tmp_dir) {
 		if (mount("tmpfs", RUN_WHITELIST_TMP_DIR, "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
 			errExit("mount tmpfs");
+		fs_logger2("tmpfs", RUN_WHITELIST_TMP_DIR);
+	}
+
+	// mask the real /var directory, currently mounted on RUN_WHITELIST_VAR_DIR
+	if (var_dir) {
+		if (mount("tmpfs", RUN_WHITELIST_VAR_DIR, "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
+			errExit("mount tmpfs");
+		fs_logger2("tmpfs", RUN_WHITELIST_VAR_DIR);
+	}
+
+	// mask the real /opt directory, currently mounted on RUN_WHITELIST_OPT_DIR
+	if (opt_dir) {
+		if (mount("tmpfs", RUN_WHITELIST_OPT_DIR, "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
+			errExit("mount tmpfs");
+		fs_logger2("tmpfs", RUN_WHITELIST_OPT_DIR);
+	}
+
+	// mask the real /dev directory, currently mounted on RUN_WHITELIST_DEV_DIR
+	if (dev_dir) {
+		if (mount("tmpfs", RUN_WHITELIST_DEV_DIR, "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
+			errExit("mount tmpfs");
+		fs_logger2("tmpfs", RUN_WHITELIST_DEV_DIR);
+	}
+
+	// mask the real /media directory, currently mounted on RUN_WHITELIST_MEDIA_DIR
+	if (media_dir) {
+		if (mount("tmpfs", RUN_WHITELIST_MEDIA_DIR, "tmpfs", MS_NOSUID | MS_STRICTATIME | MS_REC,  "mode=755,gid=0") < 0)
+			errExit("mount tmpfs");
+		fs_logger2("tmpfs", RUN_WHITELIST_MEDIA_DIR);
 	}
 
 	if (new_name)
