@@ -30,15 +30,21 @@ static char *client_filter =
 ":FORWARD DROP [0:0]\n"
 ":OUTPUT ACCEPT [0:0]\n"
 "-A INPUT -i lo -j ACCEPT\n"
+"-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT\n"
 "# echo replay is handled by -m state RELATED/ESTABLISHED below\n"
 "#-A INPUT -p icmp --icmp-type echo-reply -j ACCEPT\n"
-"-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT\n"
 "-A INPUT -p icmp --icmp-type destination-unreachable -j ACCEPT\n"
 "-A INPUT -p icmp --icmp-type time-exceeded -j ACCEPT\n"
 "-A INPUT -p icmp --icmp-type echo-request -j ACCEPT \n"
+"# disable STUN\n"
+"-A OUTPUT -p udp --dport 3478 -j DROP\n"
+"-A OUTPUT -p udp --dport 3479 -j DROP\n"
+"-A OUTPUT -p tcp --dport 3478 -j DROP\n"
+"-A OUTPUT -p tcp --dport 3479 -j DROP\n"
 "COMMIT\n";
 
 void check_netfilter_file(const char *fname) {
+	EUID_ASSERT();
 	invalid_filename(fname);
 	
 	if (is_dir(fname) || is_link(fname) || strstr(fname, "..")) {
@@ -133,7 +139,6 @@ void netfilter(const char *fname) {
 			exit(1);
 		}
 		dup2(fd,STDIN_FILENO);
-		close(fd);
 
 		// wipe out environment variables
 		environ = NULL;
@@ -149,6 +154,11 @@ void netfilter(const char *fname) {
 		if (child < 0)
 			errExit("fork");
 		if (child == 0) {
+			// elevate privileges in order to get grsecurity working
+			if (setreuid(0, 0))
+				errExit("setreuid");
+			if (setregid(0, 0))
+				errExit("setregid");
 			environ = NULL;
 			execl(iptables, iptables, "-vL", NULL);
 			// it will never get here!!!
@@ -240,7 +250,6 @@ void netfilter6(const char *fname) {
 			exit(1);
 		}
 		dup2(fd,STDIN_FILENO);
-		close(fd);
 
 		// wipe out environment variables
 		environ = NULL;
