@@ -165,6 +165,7 @@ void check_default_gw(uint32_t defaultgw) {
 }
 
 void net_check_cfg(void) {
+	EUID_ASSERT();
 	int net_configured = 0;
 	if (cfg.bridge0.configured)
 		net_configured++;
@@ -211,7 +212,10 @@ void net_check_cfg(void) {
 		// first network is a mac device
 		else {
 			// get the host default gw
+			EUID_ROOT();	// rise permissions for grsecurity
+			// Error fopen:network_get_defaultgw(479): Permission denied
 			uint32_t gw = network_get_defaultgw();
+			EUID_USER();
 			// check the gateway is network range
 			if (in_netrange(gw, cfg.bridge0.ip, cfg.bridge0.mask))
 				gw = 0;
@@ -223,6 +227,7 @@ void net_check_cfg(void) {
 
 
 void net_dns_print_name(const char *name) {
+	EUID_ASSERT();
 	if (!name || strlen(name) == 0) {
 		fprintf(stderr, "Error: invalid sandbox name\n");
 		exit(1);
@@ -238,16 +243,14 @@ void net_dns_print_name(const char *name) {
 
 #define MAXBUF 4096
 void net_dns_print(pid_t pid) {
+	EUID_ASSERT();
 	// drop privileges - will not be able to read /etc/resolv.conf for --noroot option
-//	drop_privs(1);
 
 	// if the pid is that of a firejail  process, use the pid of the first child process
+	EUID_ROOT();
 	char *comm = pid_proc_comm(pid);
+	EUID_USER();
 	if (comm) {
-		// remove \n
-		char *ptr = strchr(comm, '\n');
-		if (ptr)
-			*ptr = '\0';
 		if (strcmp(comm, "firejail") == 0) {
 			pid_t child;
 			if (find_child(pid, &child) == 0) {
@@ -258,6 +261,7 @@ void net_dns_print(pid_t pid) {
 	}
 	
 	char *fname;
+	EUID_ROOT();
 	if (asprintf(&fname, "/proc/%d/root/etc/resolv.conf", pid) == -1)
 		errExit("asprintf");
 
@@ -276,3 +280,49 @@ void net_dns_print(pid_t pid) {
 	free(fname);
 	exit(0);
 }
+
+void network_main(pid_t child) {
+	// create veth pair or macvlan device
+	if (cfg.bridge0.configured) {
+		if (cfg.bridge0.macvlan == 0) {
+			net_configure_veth_pair(&cfg.bridge0, "eth0", child);
+		}
+		else
+			net_create_macvlan(cfg.bridge0.devsandbox, cfg.bridge0.dev, child);
+	}
+	
+	if (cfg.bridge1.configured) {
+		if (cfg.bridge1.macvlan == 0)
+			net_configure_veth_pair(&cfg.bridge1, "eth1", child);
+		else
+			net_create_macvlan(cfg.bridge1.devsandbox, cfg.bridge1.dev, child);
+	}
+	
+	if (cfg.bridge2.configured) {
+		if (cfg.bridge2.macvlan == 0)
+			net_configure_veth_pair(&cfg.bridge2, "eth2", child);
+		else
+			net_create_macvlan(cfg.bridge2.devsandbox, cfg.bridge2.dev, child);
+	}
+	
+	if (cfg.bridge3.configured) {
+		if (cfg.bridge3.macvlan == 0)
+			net_configure_veth_pair(&cfg.bridge3, "eth3", child);
+		else
+			net_create_macvlan(cfg.bridge3.devsandbox, cfg.bridge3.dev, child);
+	}
+
+	// move interfaces in sandbox
+	if (cfg.interface0.configured) {
+		net_move_interface(cfg.interface0.dev, child);
+	}
+	if (cfg.interface1.configured) {
+		net_move_interface(cfg.interface1.dev, child);
+	}
+	if (cfg.interface2.configured) {
+		net_move_interface(cfg.interface2.dev, child);
+	}
+	if (cfg.interface3.configured) {
+		net_move_interface(cfg.interface3.dev, child);
+	}
+}
