@@ -27,12 +27,62 @@
 #include <sys/resource.h>
 #include <sys/types.h>
 #include <dirent.h>
+#include <errno.h>
 
 #include <sched.h>
 #ifndef CLONE_NEWUSER
 #define CLONE_NEWUSER	0x10000000
 #endif
 
+#include <sys/prctl.h>
+#ifndef PR_SET_NO_NEW_PRIVS
+# define PR_SET_NO_NEW_PRIVS 38
+#endif
+
+
+
+static int monitored_pid = 0;
+static void sandbox_handler(int sig){
+	if (!arg_quiet) {
+		printf("\nChild received signal %d, shutting down the sandbox...\n", sig);
+		fflush(0);
+	}
+
+	// broadcast sigterm to all processes in the group
+	kill(-1, SIGTERM);
+	sleep(1);
+
+	if (monitored_pid) {
+		int monsec = 9;
+		char *monfile;
+		if (asprintf(&monfile, "/proc/%d/cmdline", monitored_pid) == -1)
+			errExit("asprintf");
+		while (monsec) {
+			FILE *fp = fopen(monfile, "r");
+			if (!fp)
+				break;
+				
+			char c;
+			size_t count = fread(&c, 1, 1, fp);
+			fclose(fp);
+			if (count == 0)
+				break;
+
+			if (arg_debug)
+				printf("Waiting on PID %d to finish\n", monitored_pid);
+			sleep(1);
+			monsec--;
+		}
+		free(monfile);
+		
+	}
+
+	// broadcast a SIGKILL
+	kill(-1, SIGKILL);
+	exit(sig);
+}
+
+
 static void set_caps(void) {
 	if (arg_caps_drop_all)
 		caps_drop_all();
@@ -129,20 +179,31 @@ static void chk_chroot(void) {
 	exit(1);
 }
 
-static void monitor_application(pid_t app_pid) {
-	while (app_pid) {
+static int monitor_application(pid_t app_pid) {
+	monitored_pid = app_pid;
+	signal (SIGTERM, sandbox_handler);
+	EUID_USER();
+
+	int status = 0;
+	while (monitored_pid) {
 		usleep(20000);
+		char *msg;
+		if (asprintf(&msg, "monitoring pid %d\n", monitored_pid) == -1)
+			errExit("asprintf");
+		logmsg(msg);
+		if (arg_debug)
+			printf("%s\n", msg);
+		free(msg);
 
-		int status;
 		pid_t rv;
 		do {
 			rv = waitpid(-1, &status, 0);
 			if (rv == -1)
 				break;
 		}
-		while(rv != app_pid);
+		while(rv != monitored_pid);
 		if (arg_debug)
-			printf("Sandbox monitor: waitpid %u retval %d status %d\n", app_pid, rv, status);
+			printf("Sandbox monitor: waitpid %u retval %d status %d\n", monitored_pid, rv, status);
 
 		DIR *dir;
 		if (!(dir = opendir("/proc"))) {
@@ -155,34 +216,64 @@ static void monitor_application(pid_t app_pid) {
 		}
 
 		struct dirent *entry;
-		app_pid = 0;
+		monitored_pid = 0;
 		while ((entry = readdir(dir)) != NULL) {
 			unsigned pid;
 			if (sscanf(entry->d_name, "%u", &pid) != 1)
 				continue;
 			if (pid == 1)
 				continue;
-			app_pid = pid;
+			
+			// todo: make this generic
+			// Dillo browser leaves a dpid process running, we need to shut it down
+			if (strcmp(cfg.command_name, "dillo") == 0) {
+				char *pidname = pid_proc_comm(pid);
+				if (pidname && strcmp(pidname, "dpid") == 0)
+					break;
+				free(pidname);
+			}
+
+			monitored_pid = pid;
 			break;
 		}
 		closedir(dir);
 
-		if (app_pid != 0 && arg_debug)
-			printf("Sandbox monitor: monitoring %u\n", app_pid);
+		if (monitored_pid != 0 && arg_debug)
+			printf("Sandbox monitor: monitoring %u\n", monitored_pid);
 	}
 
+	// return the latest exit status.
+	return status;
+
+#if 0
+// todo: find a way to shut down interfaces before closing the namespace
+// the problem is we don't have enough privileges to shutdown interfaces in this moment
+	// shut down bridge/macvlan interfaces
+	if (any_bridge_configured()) {
+		
+		if (cfg.bridge0.configured) {
+			printf("Shutting down %s\n", cfg.bridge0.devsandbox);
+			net_if_down( cfg.bridge0.devsandbox);
+		}
+		if (cfg.bridge1.configured) {
+			printf("Shutting down %s\n", cfg.bridge1.devsandbox);
+			net_if_down( cfg.bridge1.devsandbox);
+		}
+		if (cfg.bridge2.configured) {
+			printf("Shutting down %s\n", cfg.bridge2.devsandbox);
+			net_if_down( cfg.bridge2.devsandbox);
+		}
+		if (cfg.bridge3.configured) {
+			printf("Shutting down %s\n", cfg.bridge3.devsandbox);
+			net_if_down( cfg.bridge3.devsandbox);
+		}
+		usleep(20000);	// 20 ms sleep
+	}	
+#endif	
 }
 
 
 static void start_application(void) {
-	// set environment
-	env_defaults();
-	env_apply();
-	if (arg_debug) {
-		printf("starting application\n");
-		printf("LD_PRELOAD=%s\n", getenv("LD_PRELOAD"));
-	}
-	
 	//****************************************
 	// start the program without using a shell
 	//****************************************
@@ -196,8 +287,14 @@ static void start_application(void) {
 			}
 		}
 
+		if (cfg.original_program_index == 0) {
+			fprintf(stderr, "Error: --shell=none configured, but no program specified\n");
+			exit(1);
+		}
+
 		if (!arg_command && !arg_quiet)
 			printf("Child process initialized\n");
+
 		execvp(cfg.original_argv[cfg.original_program_index], &cfg.original_argv[cfg.original_program_index]);
 	}
 	//****************************************
@@ -338,6 +435,9 @@ int sandbox(void* sandbox_arg) {
 #ifdef HAVE_CHROOT		
 	if (cfg.chrootdir) {
 		fs_chroot(cfg.chrootdir);
+		// redo cp command
+		fs_build_cp_command();
+		
 		// force caps and seccomp if not started as root
 		if (getuid() != 0) {
 			// force default seccomp inside the chroot, no keep or drop list
@@ -403,21 +503,13 @@ int sandbox(void* sandbox_arg) {
 	if (arg_private_dev)
 		fs_private_dev();
 	if (arg_private_etc) {
-		if (cfg.chrootdir)
-			fprintf(stderr, "Warning: private-etc feature is disabled in chroot\n");
-		else {
-			fs_private_etc_list();
-			// create /etc/ld.so.preload file again
-			if (arg_trace || arg_tracelog)
-				fs_trace_preload();
-		}
-	}
-	if (arg_private_bin) {
-		if (cfg.chrootdir)
-			fprintf(stderr, "Warning: private-bin feature is disabled in chroot\n");
-		else
-			fs_private_bin_list();
+		fs_private_etc_list();
+		// create /etc/ld.so.preload file again
+		if (arg_trace || arg_tracelog)
+			fs_trace_preload();
 	}
+	if (arg_private_bin)
+		fs_private_bin_list();
 	if (arg_private_tmp)
 		fs_private_tmp();
 	
@@ -454,6 +546,7 @@ int sandbox(void* sandbox_arg) {
 	//****************************
 	// networking
 	//****************************
+	int gw_cfg_failed = 0; // default gw configuration flag
 	if (arg_nonetwork) {
 		net_if_up("lo");
 		if (arg_debug)
@@ -479,13 +572,6 @@ int sandbox(void* sandbox_arg) {
 			net_config_mac(cfg.bridge3.devsandbox, cfg.bridge3.macsandbox);
 		sandbox_if_up(&cfg.bridge3);
 		
-		// add a default route
-		if (cfg.defaultgw) {
-			// set the default route
-			if (net_add_route(0, 0, cfg.defaultgw))
-				fprintf(stderr, "Warning: cannot configure default route\n");
-		}
-		
 		// enable interfaces
 		if (cfg.interface0.configured && cfg.interface0.ip) {
 			if (arg_debug)
@@ -512,6 +598,15 @@ int sandbox(void* sandbox_arg) {
 			net_if_up(cfg.interface3.dev);
 		}			
 			
+		// add a default route
+		if (cfg.defaultgw) {
+			// set the default route
+			if (net_add_route(0, 0, cfg.defaultgw)) {
+				fprintf(stderr, "Warning: cannot configure default route\n");
+				gw_cfg_failed = 1;
+			}
+		}
+
 		if (arg_debug)
 			printf("Network namespace enabled\n");
 	}
@@ -527,8 +622,12 @@ int sandbox(void* sandbox_arg) {
 			printf("\n");
 			if (any_bridge_configured() || any_interface_configured())
 				net_ifprint();
-			if (cfg.defaultgw != 0)
-				printf("Default gateway %d.%d.%d.%d\n", PRINT_IP(cfg.defaultgw));
+			if (cfg.defaultgw != 0) {
+				if (gw_cfg_failed)
+					printf("Default gateway configuration failed\n");
+				else
+					printf("Default gateway %d.%d.%d.%d\n", PRINT_IP(cfg.defaultgw));
+			}
 			if (cfg.dns1 != 0)
 				printf("DNS server %d.%d.%d.%d\n", PRINT_IP(cfg.dns1));
 			if (cfg.dns2 != 0)
@@ -564,11 +663,31 @@ int sandbox(void* sandbox_arg) {
 		}
 	}
 	
+	// set environment
+	env_defaults();
+	
+	// set user-supplied environment variables
+	env_apply();
+
+	// set nice
+	if (arg_nice) {
+		errno = 0;
+		int rv = nice(cfg.nice);
+		(void) rv;
+		if (errno) {
+			fprintf(stderr, "Warning: cannot set nice value\n");
+			errno = 0;
+		}
+	}
+	
+	// clean /tmp/.X11-unix sockets
+	fs_x11();
+	
 	//****************************
 	// set security filters
 	//****************************
 	// set capabilities
-	if (!arg_noroot)
+//	if (!arg_noroot)
 		set_caps();
 
 	// set rlimits
@@ -610,14 +729,14 @@ int sandbox(void* sandbox_arg) {
 	if (arg_noroot) {
 		int rv = unshare(CLONE_NEWUSER);
 		if (rv == -1) {
-			fprintf(stderr, "Error: cannot mount a new user namespace\n");
-			perror("unshare");
+			fprintf(stderr, "Warning: cannot create a new user namespace, going forward without it...\n");
 			drop_privs(arg_nogroups);
+			arg_noroot = 0;
 		}
 	}
 	else
 		drop_privs(arg_nogroups);
- 	
+	
 	// notify parent that new user namespace has been created so a proper
  	// UID/GID map can be setup
  	notify_other(child_to_parent_fds[1]);
@@ -630,11 +749,22 @@ int sandbox(void* sandbox_arg) {
 	// somehow, the new user namespace resets capabilities;
 	// we need to do them again
 	if (arg_noroot) {
-		set_caps();
 		if (arg_debug)
 			printf("noroot user namespace installed\n");
+		set_caps();
 	}
+	
+	//****************************************
+	// Set NO_NEW_PRIVS if desired
+	//****************************************
+	if (arg_nonewprivs) {
+		int no_new_privs = prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0);
 
+		if(no_new_privs != 0)
+			fprintf(stderr, "Warning: NO_NEW_PRIVS disabled, it requires a Linux kernel version 3.5 or newer.\n");
+		else if (arg_debug)
+			printf("NO_NEW_PRIVS set\n");
+	}
 
 	//****************************************
 	// fork the application and monitor it
@@ -648,8 +778,13 @@ int sandbox(void* sandbox_arg) {
 		start_application();	// start app
 	}
 
-	monitor_application(app_pid);	// monitor application
-	flush_stdin();
-	
-	return 0;
+	int status = monitor_application(app_pid);	// monitor application
+
+	if (WIFEXITED(status)) {
+		// if we had a proper exit, return that exit status
+		return WEXITSTATUS(status);
+	} else {
+		// something else went wrong!
+		return -1;
+	}
 }
