@@ -101,22 +101,10 @@ static void filter_init(void) {
 	sfilter_alloc_size = SECSIZE;
 	
 	// copy the start entries
-#if defined(__x86_64__)
-#define X32_SYSCALL_BIT 0x40000000
-	struct sock_filter filter[] = {
-		VALIDATE_ARCHITECTURE,
-		EXAMINE_SYSCALL,
-		// handle X32 ABI
-		BPF_JUMP(BPF_JMP+BPF_JGE+BPF_K, X32_SYSCALL_BIT, 1, 0),
-		BPF_JUMP(BPF_JMP+BPF_JGE+BPF_K, 0, 1, 0),
-		RETURN_ERRNO(EPERM)
-	};
-#else
 	struct sock_filter filter[] = {
 		VALIDATE_ARCHITECTURE,
 		EXAMINE_SYSCALL
 	};
-#endif
 	sfilter_index = sizeof(filter) / sizeof(struct sock_filter);	
 	memcpy(sfilter, filter, sizeof(filter));
 }
@@ -273,7 +261,7 @@ static void filter_end_whitelist(void) {
 }
 
 
-// save seccomp filter in  /tmp/firejail/mnt/seccomp
+// save seccomp filter in  /run/firejail/mnt/seccomp
 static void write_seccomp_file(void) {
 	fs_build_mnt_dir();
 	assert(sfilter);
@@ -295,15 +283,15 @@ static void write_seccomp_file(void) {
 		errExit("chown");
 }
 
-// read seccomp filter from /tmp/firejail/mnt/seccomp
+// read seccomp filter from /run/firejail/mnt/seccomp
 static void read_seccomp_file(const char *fname) {
 	assert(sfilter == NULL && sfilter_index == 0);
 
 	// check file
 	struct stat s;
 	if (stat(fname, &s) == -1) {
-		fprintf(stderr, "Error: seccomp file not found\n");
-		exit(1);
+		fprintf(stderr, "Warning: seccomp file not found\n");
+		return;
 	}
 	ssize_t sz = s.st_size;
 	if (sz == 0 || (sz % sizeof(struct sock_filter)) != 0) {
@@ -336,7 +324,7 @@ static void read_seccomp_file(const char *fname) {
 		filter_debug();
 }
 
-// i386t filter installed on amd64 architectures
+// i386 filter installed on amd64 architectures
 void seccomp_filter_32(void) {
 	// hardcoded syscall values
 	struct sock_filter filter[] = {
@@ -385,6 +373,9 @@ void seccomp_filter_32(void) {
 		BLACKLIST(317), // move_pages
 		BLACKLIST(316), // vmsplice
 		BLACKLIST(61),  // chroot
+		BLACKLIST(88), // reboot
+		BLACKLIST(169), // nfsservctl
+		BLACKLIST(130), // get_kernel_syms
 		RETURN_ALLOW
 	};
 
@@ -401,6 +392,76 @@ void seccomp_filter_32(void) {
 	}
 }
 
+// amd64 filter installed on i386 architectures
+void seccomp_filter_64(void) {
+	// hardcoded syscall values
+	struct sock_filter filter[] = {
+		VALIDATE_ARCHITECTURE_64,
+		EXAMINE_SYSCALL,
+		BLACKLIST(165), // mount
+		BLACKLIST(166), // umount2
+		BLACKLIST(101), // ptrace
+		BLACKLIST(246), // kexec_load
+		BLACKLIST(304), // open_by_handle_at
+		BLACKLIST(175), // init_module
+		BLACKLIST(313), // finit_module
+		BLACKLIST(176), // delete_module
+		BLACKLIST(172), // iopl
+		BLACKLIST(173), // ioperm
+		BLACKLIST(167), // swapon
+		BLACKLIST(168), // swapoff
+		BLACKLIST(103), // syslog
+		BLACKLIST(310), // process_vm_readv
+		BLACKLIST(311), // process_vm_writev
+		BLACKLIST(139), // sysfs
+		BLACKLIST(156), // _sysctl
+		BLACKLIST(159), // adjtimex
+		BLACKLIST(305), // clock_adjtime
+		BLACKLIST(212), // lookup_dcookie
+		BLACKLIST(298), // perf_event_open
+		BLACKLIST(300), // fanotify_init
+		BLACKLIST(312), // kcmp
+		BLACKLIST(248), // add_key
+		BLACKLIST(249), // request_key
+		BLACKLIST(250), // keyctl
+		BLACKLIST(134), // uselib
+		BLACKLIST(163), // acct
+		BLACKLIST(154), // modify_ldt
+		BLACKLIST(155), // pivot_root
+		BLACKLIST(206), // io_setup
+		BLACKLIST(207), // io_destroy
+		BLACKLIST(208), // io_getevents
+		BLACKLIST(209), // io_submit
+		BLACKLIST(210), // io_cancel
+		BLACKLIST(216), // remap_file_pages
+		BLACKLIST(237), // mbind
+		BLACKLIST(239), // get_mempolicy
+		BLACKLIST(238), // set_mempolicy
+		BLACKLIST(256), // migrate_pages
+		BLACKLIST(279), // move_pages
+		BLACKLIST(278), // vmsplice
+		BLACKLIST(161), // chroot
+		BLACKLIST(184), // tuxcall
+		BLACKLIST(169), // reboot
+		BLACKLIST(180), // nfsservctl
+		BLACKLIST(177), // get_kernel_syms
+		RETURN_ALLOW
+	};
+
+	struct sock_fprog prog = {
+		.len = (unsigned short)(sizeof(filter) / sizeof(filter[0])),
+		.filter = filter,
+	};
+
+	if (prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER, &prog) || prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0)) {
+		;
+	}
+	else if (arg_debug) {
+		printf("Dual i386/amd64 seccomp filter configured\n");
+	}
+}
+
+
 // drop filter for seccomp option
 int seccomp_filter_drop(int enforce_seccomp) {
 	filter_init();
@@ -410,6 +471,9 @@ int seccomp_filter_drop(int enforce_seccomp) {
 #if defined(__x86_64__)
 		seccomp_filter_32();
 #endif
+#if defined(__i386__)
+		seccomp_filter_64();
+#endif
 
 #ifdef SYS_mount		
 		filter_add_blacklist(SYS_mount, 0);
@@ -571,9 +635,19 @@ int seccomp_filter_drop(int enforce_seccomp) {
  // CHECK_SECCOMP(seccomp_rule_add(ctx, SCMP_ACT_ERRNO(EPERM), SCMP_SYS(clone), 1,
  //     SCMP_A0(SCMP_CMP_MASKED_EQ, CLONE_NEWUSER, CLONE_NEWUSER)));
 
-// 32bit
-//		filter_add_blacklist(SYS_personality, 0); // test wine
-//		filter_add_blacklist(SYS_set_thread_area, 0); // test wine
+// 0.9.39
+#ifdef SYS_tuxcall
+		filter_add_blacklist(SYS_tuxcall, 0);
+#endif
+#ifdef SYS_reboot
+		filter_add_blacklist(SYS_reboot, 0);
+#endif
+#ifdef SYS_nfsservctl
+		filter_add_blacklist(SYS_nfsservctl, 0);
+#endif
+#ifdef SYS_get_kernel_syms
+		filter_add_blacklist(SYS_get_kernel_syms, 0);
+#endif
 	}
 
 	// default seccomp filter with additional drop list
@@ -726,6 +800,7 @@ void seccomp_set(void) {
 }
 
 void seccomp_print_filter_name(const char *name) {
+	EUID_ASSERT();
 	if (!name || strlen(name) == 0) {
 		fprintf(stderr, "Error: invalid sandbox name\n");
 		exit(1);
@@ -740,13 +815,13 @@ void seccomp_print_filter_name(const char *name) {
 }
 
 void seccomp_print_filter(pid_t pid) {
+	EUID_ASSERT();
+
 	// if the pid is that of a firejail  process, use the pid of the first child process
+	EUID_ROOT();
 	char *comm = pid_proc_comm(pid);
+	EUID_USER();
 	if (comm) {
-		// remove \n
-		char *ptr = strchr(comm, '\n');
-		if (ptr)
-			*ptr = '\0';
 		if (strcmp(comm, "firejail") == 0) {
 			pid_t child;
 			if (find_child(pid, &child) == 0) {
@@ -766,8 +841,8 @@ void seccomp_print_filter(pid_t pid) {
 		}
 	}
 
-
 	// find the seccomp filter
+	EUID_ROOT();
 	char *fname;
 	if (asprintf(&fname, "/proc/%d/root%s", pid, RUN_SECCOMP_CFG) == -1)
 		errExit("asprintf");
