@@ -27,18 +27,25 @@
 #include <unistd.h>
 #include <arpa/inet.h>
 #include <time.h>
+#include <fcntl.h>
 #define PIDS_BUFLEN 4096
 #define SERVER_PORT 889	// 889-899 is left unassigned by IANA
 
+//#define DEBUG_PRCTL
+
 static int pid_is_firejail(pid_t pid) {
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, pid %d\n", __FUNCTION__, __LINE__, pid);
+#endif
 	uid_t rv = 0;
 	
-	// open stat file
+	// open /proc/self/comm
 	char *file;
-	if (asprintf(&file, "/proc/%u/status", pid) == -1) {
+	if (asprintf(&file, "/proc/%u/comm", pid) == -1) {
 		perror("asprintf");
 		exit(1);
 	}
+	
 	FILE *fp = fopen(file, "r");
 	if (!fp) {
 		free(file);
@@ -47,24 +54,76 @@ static int pid_is_firejail(pid_t pid) {
 
 	// look for firejail executable name
 	char buf[PIDS_BUFLEN];
-	while (fgets(buf, PIDS_BUFLEN - 1, fp)) {
-		if (strncmp(buf, "Name:", 5) == 0) {
-			char *ptr = buf + 5;
-			while (*ptr != '\0' && (*ptr == ' ' || *ptr == '\t')) {
-				ptr++;
+	if (fgets(buf, PIDS_BUFLEN - 1, fp)) {
+		if (strncmp(buf, "firejail", 8) == 0)
+			rv = 1;
+	}
+	
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, comm %s, rv %d\n", __FUNCTION__, __LINE__, buf, rv);
+#endif
+	if (rv) {
+		// open /proc/pid/cmdline file
+		char *fname;
+		int fd;
+		if (asprintf(&fname, "/proc/%d/cmdline", pid) == -1)
+			errExit("asprintf");
+		if ((fd = open(fname, O_RDONLY)) < 0) {
+			free(fname);
+			rv = 0;
+			goto doexit;
+		}
+		free(fname);
+	
+		// read file
+#define BUFLEN 4096
+		unsigned char buffer[BUFLEN];
+		ssize_t len;
+		if ((len = read(fd, buffer, sizeof(buffer) - 1)) <= 0) {
+			close(fd);
+			rv = 0;
+			goto doexit;
+		}
+		buffer[len] = '\0';
+		close(fd);
+	
+		// list of firejail arguments that don't trigger sandbox creation
+		// the initial -- is not included 
+		char *firejail_args = "ls list tree x11 help version top netstats debug-syscalls debug-errnos debug-protocols";
+		
+		int i;
+		char *start;
+		int first = 1;
+		for (i = 0; i < len; i++) {
+			if (buffer[i] != '\0')
+				continue;
+			if (first) {
+				first = 0;
+				start = (char *) buffer + i + 1;
+				continue;
 			}
-			if (*ptr == '\0')
-				goto doexit;
-			if (strncmp(ptr, "firejail", 8) == 0)
-				rv = 1;
-//			if (strncmp(ptr, "lxc-execute", 11) == 0)
-//				rv = 1;
-			break;
+			if (strncmp(start, "--", 2) != 0)
+				break;
+			
+			// clan starting with =
+			char *ptr = strchr(start + 2, '=');
+			if (ptr)
+				*ptr = '\0';
+			
+			if (strstr(firejail_args, start + 2)) {
+				rv = 0;
+				break;
+			}
+			start = (char *) buffer + i + 1;
 		}
 	}
+
 doexit:	
 	fclose(fp);
 	free(file);
+#ifdef DEBUG_PRCTL
+	printf("%s: %d: return %d\n", __FUNCTION__, __LINE__, rv);
+#endif
 	return rv;
 }
 
@@ -197,10 +256,16 @@ static int procevent_monitor(const int sock, pid_t mypid) {
 			int remove_pid = 0;
 			switch (proc_ev->what) {
 				case PROC_EVENT_FORK:
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, event fork\n", __FUNCTION__, __LINE__);
+#endif
 					if (proc_ev->event_data.fork.child_pid !=
 					    proc_ev->event_data.fork.child_tgid)
 					    	continue; // this is a thread, not a process
 					pid = proc_ev->event_data.fork.parent_tgid;
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, event fork, pid %d\n", __FUNCTION__, __LINE__, pid);
+#endif
 					if (pids[pid].level > 0) {
 						child = proc_ev->event_data.fork.child_tgid;
 						child %= max_pids;
@@ -211,6 +276,12 @@ static int procevent_monitor(const int sock, pid_t mypid) {
 					break;
 				case PROC_EVENT_EXEC:
 					pid = proc_ev->event_data.exec.process_tgid;
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, event exec, pid %d\n", __FUNCTION__, __LINE__, pid);
+#endif
+					if (pids[pid].level == -1) {
+						pids[pid].level = 0; // start tracking
+					}
 					sprintf(lineptr, " exec");
 					break;
 					
@@ -220,26 +291,41 @@ static int procevent_monitor(const int sock, pid_t mypid) {
 						continue; // this is a thread, not a process
 
 					pid = proc_ev->event_data.exit.process_tgid;
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, event exit, pid %d\n", __FUNCTION__, __LINE__, pid);
+#endif
 					remove_pid = 1;
 					sprintf(lineptr, " exit");
 					break;
 					
 				case PROC_EVENT_UID:
 					pid = proc_ev->event_data.id.process_tgid;
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, event uid, pid %d\n", __FUNCTION__, __LINE__, pid);
+#endif
 					sprintf(lineptr, " uid ");
 					break;
 
 				case PROC_EVENT_GID:
 					pid = proc_ev->event_data.id.process_tgid;
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, event gid, pid %d\n", __FUNCTION__, __LINE__, pid);
+#endif
 					sprintf(lineptr, " gid ");
 					break;
 
 				case PROC_EVENT_SID:
 					pid = proc_ev->event_data.sid.process_tgid;
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, event sid, pid %d\n", __FUNCTION__, __LINE__, pid);
+#endif
 					sprintf(lineptr, " sid ");
 					break;
 
 				default:
+#ifdef DEBUG_PRCTL
+	printf("%s: %d, event unknown\n", __FUNCTION__, __LINE__);
+#endif
 					sprintf(lineptr, "\n");
 					continue;
 			}
