@@ -58,6 +58,7 @@ int join_namespace(pid_t pid, char *type) {
 }
 
 // return 1 if error
+// this function requires root access - todo: fix it!
 int name2pid(const char *name, pid_t *pid) {
 	pid_t parent = getpid();
 	
@@ -83,10 +84,6 @@ int name2pid(const char *name, pid_t *pid) {
 		// check if this is a firejail executable
 		char *comm = pid_proc_comm(newpid);
 		if (comm) {
-			// remove \n
-			char *ptr = strchr(comm, '\n');
-			if (ptr)
-				*ptr = '\0';
 			if (strcmp(comm, "firejail")) {
 				free(comm);
 				continue;
@@ -94,53 +91,34 @@ int name2pid(const char *name, pid_t *pid) {
 			free(comm);
 		}
 		
-		char *cmd = pid_proc_cmdline(newpid);
-		if (cmd) {
-			// mark the end of the name
-			char *ptr = strstr(cmd, "--name=");
-			char *start = ptr;
-			if (!ptr) {
-				free(cmd);
-				
-				// extract name for /run/mnt/firejail/fslogger file
-				char *fname;
-				if (asprintf(&fname, "/proc/%d/root/run/firejail/mnt/fslogger", newpid) == -1)
-					errExit("asprintf");
-
-				struct stat s;
-				if (stat(fname, &s) == 0) {
-					FILE *fp = fopen(fname, "r");
-					if (fp) {
-						char buf[BUFLEN];
-						if (fgets(buf, BUFLEN, fp)) {
-							if (strncmp(buf, "sandbox name: ", 14) == 0) {
-								char *ptr2 = buf + 14;
-								if (strncmp(name, ptr2, strlen(name)) == 0) {
-									fclose(fp);
-									*pid = newpid;
-									closedir(dir);
-									return 0;
-								}
-							}
-						}
+		// look for the sandbox name in /run/firejail/name/<PID>
+		// todo: use RUN_FIREJAIL_NAME_DIR define from src/firejail/firejail.h
+		char *fname;
+		if (asprintf(&fname, "/run/firejail/name/%d", newpid) == -1)
+			errExit("asprintf");
+		FILE *fp = fopen(fname, "r");
+		if (fp) {
+			char buf[BUFLEN];
+			if (fgets(buf, BUFLEN, fp)) {
+				// remove \n
+				char *ptr = strchr(buf, '\n');
+				if (ptr) {
+					*ptr = '\0';
+					if (strcmp(buf, name) == 0) {
+						// we found it!
 						fclose(fp);
+						free(fname);
+						*pid = newpid;
+						closedir(dir);
+						return 0;
 					}
-				}	
-				
-				continue;
+				}
+				else
+					fprintf(stderr, "Error: invalid %s\n", fname);
 			}
-			while (*ptr != ' ' && *ptr != '\t' && *ptr != '\0')
-				ptr++;
-			*ptr = '\0';
-			int rv = strcmp(start + 7, name);
-			if (rv == 0) {
-				free(cmd);
-				*pid = newpid;
-				closedir(dir);
-				return 0;
-			}
-			free(cmd);
+			fclose(fp);
 		}
+		free(fname);
 	}
 	closedir(dir);
 	return 1;
@@ -159,7 +137,7 @@ char *pid_proc_comm(const pid_t pid) {
 	free(fname);
 
 	// read file
-	unsigned char buffer[BUFLEN];
+	char buffer[BUFLEN];
 	ssize_t len;
 	if ((len = read(fd, buffer, sizeof(buffer) - 1)) <= 0) {
 		close(fd);
@@ -168,8 +146,13 @@ char *pid_proc_comm(const pid_t pid) {
 	buffer[len] = '\0';
 	close(fd);
 
+	// remove \n
+	char *ptr = strchr(buffer, '\n');
+	if (ptr)
+		*ptr = '\0';
+
 	// return a malloc copy of the command line
-	char *rv = strdup((char *) buffer);
+	char *rv = strdup(buffer);
 	if (strlen(rv) == 0) {
 		free(rv);
 		return NULL;
